/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * JBIListener.java - Initial Developper : ofabre
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.validation.listener;

import java.util.logging.Level;

import javax.xml.namespace.QName;
import javax.xml.validation.Schema;

import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.se.validation.Constants;
import org.ow2.petals.se.validation.ValidationComponent;
import org.ow2.petals.se.validation.service.ValidationService;

/**
 * The VALIDATION service engine JBIListener
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author mbeldame
 * 
 */
public class JBIListener extends AbstractJBIListener {

    private ValidationService validationService;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.petals.component.framework.listener.AbstractListener#init()
     */
    @Override
    public void init() {
        this.validationService = new ValidationService(this.getLogger());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.petals.component.framework.listener.AbstractJBIListener
     * #onJBIMessage(org.objectweb.petals.component.framework.util.Exchange)
     */
    @Override
    public boolean onJBIMessage(final Exchange exchange) {

        if (!validateExchange(exchange)) {
            this.getLogger().log(Level.WARNING, exchange.getError().getMessage());
            // no SQL request, error is set in the exchange
            return true;
        }

        try {
            // get the XSD document for current Service Unit
            final Schema xsdSchema = ((ValidationComponent) this.component).getSchemas().get(
                    exchange.getEndpointName());

            if (Constants.OPERATION_VALIDATE.equals(exchange.getOperation().getLocalPart())) {
                this.validationService.validate(exchange, xsdSchema);
            } else {
                this.validationService.filter(exchange, xsdSchema);
            }
        } catch (final Exception e) {
            this.getLogger().log(Level.WARNING,
                    e.getClass().getSimpleName() + " : " + e.getMessage());
            // technical error, it would be set as a Fault by the CDK
            if (!exchange.isActiveStatus()) {
                exchange.setError(new Exception(e));
            }
        }

        return true;
    }

    /**
     * Validate the operation and the MEP.
     * 
     * @param operation
     */
    private static final boolean validateExchange(final Exchange exchange) {
        boolean result = true;

        final QName operation = exchange.getOperation();
        if (!operation.getNamespaceURI().equals(Constants.VALIDATION_NS)) {
            exchange.setError(new Exception("Operation '" + operation
                    + "' must be in the namespace '" + Constants.VALIDATION_NS + "'"));
            result = false;
        }

        if (Constants.OPERATION_VALIDATE.equals(operation.getLocalPart().toLowerCase())) {
            if (!Message.MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                exchange.setError(new Exception("Operation '" + operation
                        + "' must use the pattern '" + Message.MEPConstants.IN_OUT_PATTERN.value()
                        + "'"));
                result = false;
            }
        } else if (Constants.OPERATION_FILTER.equals(operation.getLocalPart().toLowerCase())) {
            if (!Message.MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                exchange.setError(new Exception("Operation '" + operation
                        + "' must use the pattern '" + Message.MEPConstants.IN_OUT_PATTERN.value()
                        + "'"));
                result = false;
            }
        } else {
            exchange.setError(new Exception("Operation '" + operation
                    + "' must have a declared local name. Choices are :'"
                    + Constants.OPERATION_VALIDATE + "','" + Constants.OPERATION_FILTER + "'"));
            result = false;
        }

        return result;
    }

}
